#include <avr/io.h>
#include <util/setbaud.h>

void hw_init( void )
{
  DDRD|=(1<<PD6)|(1<<PD5);        // Init PD5 and PD6 pins as output for servo
}

void uart_init( void ) 
{
    UBRR0H = UBRRH_VALUE;
    UBRR0L = UBRRL_VALUE;

#if USE_2X
    UCSR0A |= _BV(U2X0);
#else
    UCSR0A &= ~(_BV(U2X0));
#endif

    UCSR0C = _BV(UCSZ01) | _BV(UCSZ00); /* 8-bit data */
    UCSR0B = _BV(RXEN0) | _BV(TXEN0);   /* Enable RX and TX */
}


void uart_putc(char c) 
{
   if (c == '\n') 
    {
       uart_putc('\r');  // tu nemusi byt cakanie, lebo rekurzivne to zavola dalsia
    }
   loop_until_bit_is_set(UCSR0A, UDRE0); /* Wait until data register empty. */
   UDR0 = c;
}



void uart_puts(const char *s)
{
	register unsigned char c;
	while((c = *s++))	
	    uart_putc(c); 
}

char uart_getc(void) {
    loop_until_bit_is_set(UCSR0A, RXC0); /* Wait until data exists. */
    return UDR0;
}


void adc_init(void)
{
    ADMUX = (1<<REFS0);                             // AVCC - nastavenie zdroja ref. napatia

  // sampling rate is [ADC clock] / [prescaler] / [conversion clock cycles]
  // for Arduino Uno ADC clock is 16 MHz and a conversion takes 13 clock cycles
  
  // ADCSRA |= (1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);  // nastavenie preddelica frekvencie najpomalsie
  // ADCSRA |= (1 << ADPS2) | (1 << ADPS0);       // 32 prescaler for 38.5 KHz
  //    ADCSRA |= (1 << ADPS2);                      // 16 prescaler for 76.9 KHz
   ADCSRA |= (1 << ADPS1) | (1 << ADPS0);       // 8 prescaler for 153.8 KHz		 
 
  
  // ADCSRA |= (1 << ADATE); // enable auto trigger
  // ADCSRA |= (1 << ADIE);  // enable interrupts when measurement complete
  // ADCSRA |= (1 << ADSC);  // start ADC measurements - treba nam to?
  
   ADCSRA |= (1<<ADEN);                              // "zapnutie" ADC - enable ADC, asi by malo byt az na konci
 		 		 
}

unsigned int adc_read(char a_pin)
{
   ADMUX   = (ADMUX & 0xF8)|a_pin;
   ADCSRA |= (1<<ADSC);                      // spustenie prevodu

  while(ADCSRA & (1<<ADSC));                 // pockam na dokoncenie prevodu
 
  return (ADC);
}


 //Initialize Timer0
void timer0_init(void)
{
  
  TCNT0=0;                        // Set Initial Timer value

  OCR0A=15;                       // Set Initial Pulse width
  OCR0B=31;                       // for both outputs

                                 //Set fast PWM mode + clear OC0A and set OC0B on compare match

  TCCR0A|=(1<<COM0A1)|(1<<COM0B1)|(1<<WGM01)|(1<<WGM00);

  TCCR0B|=(1<<CS02)|(1<<CS00);      // Set prescaller 1024 and start timer
}

 //Initialize Timer1
void timer1_init(void)
{  
  TCNT1=0;                         // Set Initial Timer value

  DDRB |= (1<<PB2)|(1<<PB1);       // Set outputs
  OCR0A=350;                       // Set Initial Pulse width
  OCR0B=350;                       // for both outputs

  TCCR1A|=(1<<COM1A1)|(1<<COM1B1)|(1<<WGM11);         // Set Non Inverted PWM (mode 14)
  TCCR1B|=(1<<WGM13)|(1<<WGM12)|(1<<CS10)|(1<<CS11);  // Set prescaller 64 and start timer
}
